(ns cubes.core
  (:require [quil.core :as q]
            [quil.middleware :as m]))

(defn settings []
  (q/smooth))

(defn setup []
  ; Set frame rate to 30 frames per second.
  (q/frame-rate 30)
  ; Set color mode to HSB (HSV) instead of default RGB.
  (q/color-mode :hsb)
  (q/background 0)
  (q/rect-mode :center)
  ; setup function returns initial state. It contains
  ; circle color and position.
  {:color 0
   :angle 0})

(defn update-state [state]
  ; Update sketch state by changing circle color and position.
  {:color (mod (+ (:color state) 0.7) 255)
   :angle (+ (:angle state) 0.1)})

(def w 50)
(def h 200)
(def offset (atom 0))

(defn draw-state [state]
  (q/background 0)
  (q/fill (:color state) 255 255)
  (reset! offset 0)
  (q/with-translation [(/ (q/width) 2)
                       (/ (q/height) 2)]
    (doseq [i (range 0 (q/width) w)]
      (let [angle (:angle state)
            sin (q/sin (+ angle @offset))
            cos (q/cos angle)
            h (q/map-range sin -1 1 20 h)]
        (q/rect (- (/ w 2) (- (/ (q/width) 2) i)) 0 (- w 2) h)
        (swap! offset + 1.5)))))

(q/defsketch cubes
  :title "You spin my circle right round"
  :size [500 500]
  ; setup function called only once, during sketch initialization.
  ; :renderer :opengl
  :setup setup
  :settings settings
  ; update-state is called on each iteration before draw-state.
  :update update-state
  :draw draw-state
  :features [:keep-on-top]
  ; This sketch uses functional-mode middleware.
  ; Check quil wiki for more info about middlewares and particularly
  ; fun-mode.
  :middleware [m/fun-mode])
