(ns penrose-tile.core
  (:require [quil.core :as q]
            [quil.middleware :as m]))

(def axiom "F")
(def rule "F+F-F")
(def start-length 190)
(def theta (q/radians 120))
(def production (atom (str axiom)))
(def draw-length (atom start-length))
(def generations (atom 0))
(def steps (atom 0))


(defn update-step
  []
  (if (> @steps (count @production)) 
    (reset! steps (count @production))) 
    (swap! steps (partial + 5)))

(defn iter 
  []
  (swap! draw-length (partial * 0.6))
  (swap! generations inc)
  (swap! production #(clojure.string/replace % "F" rule)))

(defn simulate
  [gen]
  (while (< @generations gen)
    (iter)))

(defn parse-string 
  "Draw the entire LSystem by parsing production"
  []
  (update-step)
  (q/translate (/ (q/width) 2) (/ (q/height) 2))
  (doseq [step (vec @production)]
    (cond 
      (= step \F) ((q/rect 0 0 (- @draw-length) (- @draw-length))
                    (q/translate 0 (- @draw-length)))
      (= step \+) (q/rotate @theta)
      (= step \-) (q/rotate (- @theta))
      (= step \[) (q/push-matrix)
      (= step \]) (q/pop-matrix))))


(defn draw-state []
  ; Set circle color.
  ;(q/background 255)
  (q/stroke 0)
  ;(q/rect (/ (q/width) 2) (/ (q/height) 2) 50 50)
  (parse-string)
  )

(defn setup []
  (q/frame-rate 30)
  (q/color-mode :hsb)
  (simulate 6))

(q/defsketch penrose-tile
  :title "You spin my circle right round"
  :size [500 500]
  ; setup function called only once, during sketch initialization.
  :setup setup
  ; update-state is called on each iteration before draw-state.
  :update identity
  :draw draw-state
  :features [:keep-on-top]
  ; This sketch uses functional-mode middleware.
  ; Check quil wiki for more info about middlewares and particularly
  ; fun-mode.
  ;:middleware [m/fun-mode])
  )
