(ns sinousidal.core
  (:require [quil.core :as q]
            [quil.middleware :as m]))

(def width 500)
(def height 500)
(def left-x (int (* width -0.5)))
(def right-x (int (* width 1.5)))
(def top-y (int (* height -0.5)))
(def bottom-y (int (* height 1.5)))
(def resolution (int (* width 0.01)))
(def num-columns (/ (- right-x left-x) resolution))
(def num-rows (/ (- bottom-y top-y) resolution))

(println left-x)

(defn setup []
  ; Set frame rate to 30 frames per second.
  (q/frame-rate 30)
  ; Set color mode to HSB (HSV) instead of default RGB.
  (q/color-mode :hsb)
  ; setup function returns initial state. It contains
  ; circle color and position.
  (let [grid (into []
                   (for [x (range num-columns)
                         :let [angle (* (/ x num-rows) q/PI)]]
                     (vec (repeat num-rows angle))))]
    {:grid grid}))

(defn update-state [state]
  state)

(defn draw-state [state]

  (when false
    (q/background 255)
    (q/stroke 0)
    (q/begin-shape)
    (let [x 500
          y 100
          steps 12
          length 20]
      (doseq [n (range steps)]
        (let [x-offset (- x (* n left-x))
              y-offset (- y (* n top-y))
              col-i (int (/ x-offset resolution))
              row-i (int (/ y-offset resolution))
              angle (-> (:grid state)
                        (nth col-i)
                        (nth row-i))
              x-step (* length (q/cos angle))
              y-step (* length (q/sin angle))]
          (println (count (:grid state)))
          (q/curve-vertex (- x (* n x-step)) (+ y (* n y-step))))))
    (q/end-shape)))

(q/defsketch sinousidal
  :title "You spin my circle right round"
  :size [width height]
  ; setup function called only once, during sketch initialization.
  :setup setup
  ; update-state is called on each iteration before draw-state.
  :update update-state
  :draw draw-state
  :features [:keep-on-top]
  ; This sketch uses functional-mode middleware.
  ; Check quil wiki for more info about middlewares and particularly
  ; fun-mode.
  :middleware [m/fun-mode])
